@@ -1,12 +1,12 @@
 {-# LANGUAGE CPP, DeriveDataTypeable, DeriveFunctor  #-}
 {-# LANGUAGE FlexibleInstances, TypeSynonymInstances #-}
 {-# LANGUAGE PatternGuards, ScopedTypeVariables      #-}
-{-# LANGUAGE RecordWildCards                         #-}
+{-# LANGUAGE RecordWildCards, TemplateHaskell        #-}
 
 {- |
 Module:      Database.PostgreSQL.Simple.FromField
 Copyright:   (c) 2011 MailRank, Inc.
-             (c) 2011-2012 Leon P Smith
+             (c) 2011-2013 Leon P Smith
 License:     BSD3
 Maintainer:  Leon P Smith <leon@melding-monads.com>
 Stability:   experimental
@@ -31,26 +31,28 @@ might look like for a UUID type that implements the @Read@ class:
 
 @
 import Data.UUID ( UUID )
-import Database.PostgreSQL.Simple.BuiltinTypes
-                 ( BuiltinType(UUID), builtin2oid )
-import qualified Data.ByteString as B
+import Database.PostgreSQL.Simple.FromField
+       ( FromField (fromField) , typeOid, returnError, ResultError (..) )
+import Database.PostgreSQL.Simple.TypeInfo.Static (typoid, uuid)
+import qualified Data.ByteString.Char8 as B
 
 instance FromField UUID where
    fromField f mdata =
-      if typeOid f /= builtin2oid UUID
+      if typeOid f /= typoid uuid
         then returnError Incompatible f \"\"
         else case B.unpack \`fmap\` mdata of
                Nothing  -> returnError UnexpectedNull f \"\"
                Just dat ->
                   case [ x | (x,t) <- reads dat, (\"\",\"\") <- lex t ] of
                     [x] -> return x
-                    _   -> returnError ConversionError f dat
+                    _   -> returnError ConversionFailed f dat
 @
 
 Note that because PostgreSQL's @uuid@ type is built into postgres and is
 not provided by an extension,  the 'typeOid' of @uuid@ does not change and
-thus we can examine it directly.   Here,  we simply pull the type oid out
-of the static table provided by postgresql-simple.
+thus we can examine it directly.  One could hard-code the type oid,  or
+obtain it by other means, but in this case we simply pull it out of the
+static table provided by postgresql-simple.
 
 On the other hand if the type is provided by an extension,  such as
 @PostGIS@ or @hstore@,  then the 'typeOid' is not stable and can vary from
@@ -85,32 +87,35 @@ module Database.PostgreSQL.Simple.FromField
     , typeOid
     , PQ.Oid(..)
     , PQ.Format(..)
+
+    , fromJSONField
     ) where
 
 #include "MachDeps.h"
 
 import           Control.Applicative
                    ( Applicative, (<|>), (<$>), pure )
+import           Control.Concurrent.MVar (MVar, newMVar)
 import           Control.Exception (Exception)
+import qualified Data.Aeson as JSON
 import           Data.Attoparsec.Char8 hiding (Result)
-import           Data.Bits ((.&.), (.|.), shiftL)
 import           Data.ByteString (ByteString)
 import qualified Data.ByteString.Char8 as B
 import           Data.Int (Int16, Int32, Int64)
-import           Data.List (foldl')
+import           Data.IORef (IORef, newIORef)
 import           Data.Ratio (Ratio)
 import           Data.Time ( UTCTime, ZonedTime, LocalTime, Day, TimeOfDay )
 import           Data.Typeable (Typeable, typeOf)
 import           Data.Vector (Vector)
+import           Data.Vector.Mutable (IOVector)
 import qualified Data.Vector as V
-import           Data.Word (Word64)
 import           Database.PostgreSQL.Simple.Internal
-import           Database.PostgreSQL.Simple.BuiltinTypes
 import           Database.PostgreSQL.Simple.Compat
 import           Database.PostgreSQL.Simple.Ok
-import           Database.PostgreSQL.Simple.Types (Binary(..), Null(..))
-import           Database.PostgreSQL.Simple.TypeInfo as TypeInfo
-import qualified Database.PostgreSQL.Simple.TypeInfo.Static as TypeInfo
+import           Database.PostgreSQL.Simple.Types
+import           Database.PostgreSQL.Simple.TypeInfo as TI
+import qualified Database.PostgreSQL.Simple.TypeInfo.Static as TI
+import           Database.PostgreSQL.Simple.TypeInfo.Macro as TI
 import           Database.PostgreSQL.Simple.Time
 import           Database.PostgreSQL.Simple.Arrays as Arrays
 import qualified Database.PostgreSQL.LibPQ as PQ
@@ -120,6 +125,8 @@ import qualified Data.ByteString.Lazy as LB
 import qualified Data.Text as ST
 import qualified Data.Text.Encoding as ST
 import qualified Data.Text.Lazy as LT
+import           Data.UUID   (UUID)
+import qualified Data.UUID as UUID
 
 -- | Exception thrown if conversion from a SQL value to a Haskell
 -- value fails.
@@ -163,14 +170,15 @@ class FromField a where
     -- library instances,  this will usually be a single 'ResultError',  but
     -- may be a 'UnicodeException'.
     --
-    -- Implementations of 'fromField' should not retain any references to
-    -- the 'Field' nor the 'ByteString' arguments after the result has
-    -- been evaluated to WHNF.  Such a reference causes the entire
-    -- @LibPQ.'PQ.Result'@ to be retained.
+    -- Note that retaining any reference to the 'Field' argument causes
+    -- the entire @LibPQ.'PQ.Result'@ to be retained.  Thus, implementations
+    -- of 'fromField' should return results that do not refer to this value
+    -- after the result have been evaluated to WHNF.
     --
-    -- For example,  the instance for 'ByteString' uses 'B.copy' to avoid
-    -- such a reference,  and that using bytestring functions such as 'B.drop'
-    -- and 'B.takeWhile' alone will also trigger this memory leak.
+    -- Note that as of @postgresql-simple-0.4.0.0@,  the 'ByteString' value
+    -- has already been copied out of the @LibPQ.'PQ.Result'@ before it has
+    -- been passed to 'fromField'.  This is because for short strings, it's
+    -- cheaper to copy the string than to set up a finalizer.
 
 -- | Returns the data type name.  This is the preferred way of identifying
 --   types that do not have a stable type oid, such as types provided by
@@ -178,9 +186,9 @@ class FromField a where
 --
 --   More concretely,  it returns the @typname@ column associated with the
 --   type oid in the @pg_type@ table.  First, postgresql-simple will check
---   built-in, static table.   If the type oid is not there, postgresql-simple
---   will check a per-connection cache,  and then finally query the database's
---   meta-schema.
+--   the built-in, static table.   If the type oid is not there,
+--   postgresql-simple will check a per-connection cache,  and then
+--   finally query the database's meta-schema.
 
 typename :: Field -> Conversion ByteString
 typename field = typname <$> typeInfo field
@@ -227,6 +235,12 @@ tableColumn Field{..} = fromCol (unsafeDupablePerformIO (PQ.ftablecol result col
 format :: Field -> PQ.Format
 format Field{..} = unsafeDupablePerformIO (PQ.fformat result column)
 
+-- | void
+instance FromField () where
+  fromField f _bs
+     | typeOid f /= $(inlineTypoid TI.void) = returnError Incompatible f ""
+     | otherwise = pure ()
+
 -- | For dealing with null values.  Compatible with any postgresql type
 --   compatible with type @a@.  Note that the type is not checked if
 --   the value is null, although it is inadvisable to rely on this
@@ -243,7 +257,7 @@ instance FromField Null where
 -- | bool
 instance FromField Bool where
     fromField f bs
-      | typeOid f /= typoid (TypeInfo.bool) = returnError Incompatible f ""
+      | typeOid f /= $(inlineTypoid TI.bool) = returnError Incompatible f ""
       | bs == Nothing                 = returnError UnexpectedNull f ""
       | bs == Just "t"                = pure True
       | bs == Just "f"                = pure False
@@ -252,7 +266,7 @@ instance FromField Bool where
 -- | \"char\"
 instance FromField Char where
     fromField f bs =
-        if typeOid f /= typoid (TypeInfo.char)
+        if typeOid f /= $(inlineTypoid TI.char)
         then returnError Incompatible f ""
         else case bs of
                Nothing -> returnError UnexpectedNull f ""
@@ -289,30 +303,30 @@ instance FromField Integer where
 -- | int2, float4
 instance FromField Float where
     fromField = atto ok (realToFrac <$> double)
-        where ok = mkCompats [Float4,Int2]
+      where ok = $(mkCompats [TI.float4,TI.int2])
 
 -- | int2, int4, float4, float8
 instance FromField Double where
     fromField = atto ok double
-        where ok = mkCompats [Float4,Float8,Int2,Int4]
+      where ok = $(mkCompats [TI.float4,TI.float8,TI.int2,TI.int4])
 
 -- | int2, int4, float4, float8, numeric
 instance FromField (Ratio Integer) where
     fromField = atto ok rational
-        where ok = mkCompats [Float4,Float8,Int2,Int4,Numeric]
+      where ok = $(mkCompats [TI.float4,TI.float8,TI.int2,TI.int4,TI.numeric])
 
 unBinary :: Binary t -> t
 unBinary (Binary x) = x
 
 -- | bytea, name, text, \"char\", bpchar, varchar, unknown
 instance FromField SB.ByteString where
-    fromField f dat = if typeOid f == typoid TypeInfo.bytea
+    fromField f dat = if typeOid f == $(inlineTypoid TI.bytea)
                       then unBinary <$> fromField f dat
-                      else doFromField f okText' (pure . B.copy) dat
+                      else doFromField f okText' pure dat
 
 -- | oid
 instance FromField PQ.Oid where
-    fromField f dat = PQ.Oid <$> atto (mkCompat Oid) decimal f dat
+    fromField f dat = PQ.Oid <$> atto (== $(inlineTypoid TI.oid)) decimal f dat
 
 -- | bytea, name, text, \"char\", bpchar, varchar, unknown
 instance FromField LB.ByteString where
@@ -328,7 +342,7 @@ unescapeBytea f str = case unsafeDupablePerformIO (PQ.unescapeBytea str) of
 instance FromField (Binary SB.ByteString) where
     fromField f dat = case format f of
       PQ.Text   -> doFromField f okBinary (unescapeBytea f) dat
-      PQ.Binary -> doFromField f okBinary (pure . Binary . B.copy) dat
+      PQ.Binary -> doFromField f okBinary (pure . Binary) dat
 
 -- | bytea
 instance FromField (Binary LB.ByteString) where
@@ -349,44 +363,44 @@ instance FromField [Char] where
 
 -- | timestamptz
 instance FromField UTCTime where
-  fromField = ff TypeInfo.timestamptz "UTCTime" parseUTCTime
+  fromField = ff $(inlineTypoid TI.timestamptz) "UTCTime" parseUTCTime
 
 -- | timestamptz
 instance FromField ZonedTime where
-  fromField = ff TypeInfo.timestamptz "ZonedTime" parseZonedTime
+  fromField = ff $(inlineTypoid TI.timestamptz) "ZonedTime" parseZonedTime
 
 -- | timestamp
 instance FromField LocalTime where
-  fromField = ff TypeInfo.timestamp "LocalTime" parseLocalTime
+  fromField = ff $(inlineTypoid TI.timestamp) "LocalTime" parseLocalTime
 
 -- | date
 instance FromField Day where
-  fromField = ff TypeInfo.date "Day" parseDay
+  fromField = ff $(inlineTypoid TI.date) "Day" parseDay
 
 -- | time
 instance FromField TimeOfDay where
-  fromField = ff TypeInfo.time "TimeOfDay" parseTimeOfDay
+  fromField = ff $(inlineTypoid TI.time) "TimeOfDay" parseTimeOfDay
 
 -- | timestamptz
 instance FromField UTCTimestamp where
-  fromField = ff TypeInfo.timestamptz "UTCTimestamp" parseUTCTimestamp
+  fromField = ff $(inlineTypoid TI.timestamptz) "UTCTimestamp" parseUTCTimestamp
 
 -- | timestamptz
 instance FromField ZonedTimestamp where
-  fromField = ff TypeInfo.timestamptz "ZonedTimestamp" parseZonedTimestamp
+  fromField = ff $(inlineTypoid TI.timestamptz) "ZonedTimestamp" parseZonedTimestamp
 
 -- | timestamp
 instance FromField LocalTimestamp where
-  fromField = ff TypeInfo.timestamp "LocalTimestamp" parseLocalTimestamp
+  fromField = ff $(inlineTypoid TI.timestamp) "LocalTimestamp" parseLocalTimestamp
 
 -- | date
 instance FromField Date where
-  fromField = ff TypeInfo.date "Date" parseDate
+  fromField = ff $(inlineTypoid TI.date) "Date" parseDate
 
-ff :: TypeInfo -> String -> (B8.ByteString -> Either String a)
+ff :: PQ.Oid -> String -> (B8.ByteString -> Either String a)
    -> Field -> Maybe B8.ByteString -> Conversion a
-ff pgType hsType parse f mstr =
-  if typeOid f /= typoid pgType
+ff compatOid hsType parse f mstr =
+  if typeOid f /= compatOid
   then err Incompatible ""
   else case mstr of
          Nothing -> err UnexpectedNull ""
@@ -411,17 +425,17 @@ instance (FromField a, FromField b) => FromField (Either a b) where
                     <|> (Left  <$> fromField f dat)
 
 -- | any postgresql array whose elements are compatible with type @a@
-instance (FromField a, Typeable a) => FromField (Vector a) where
+instance (FromField a, Typeable a) => FromField (PGArray a) where
     fromField f mdat = do
         info <- typeInfo f
         case info of
-          TypeInfo.Array{} ->
+          TI.Array{} ->
               case mdat of
                 Nothing  -> returnError UnexpectedNull f ""
                 Just dat -> do
                    case parseOnly (fromArray info f) dat of
                      Left  err  -> returnError ConversionFailed f err
-                     Right conv -> V.fromList <$> conv
+                     Right conv -> PGArray <$> conv
           _ -> returnError Incompatible f ""
 
 fromArray :: (FromField a)
@@ -434,25 +448,83 @@ fromArray typeInfo f = sequence . (parseIt <$>) <$> array delim
       where f' | Arrays.Array _ <- item = f
                | otherwise              = fElem
 
-newtype Compat = Compat Word64
+instance (FromField a, Typeable a) => FromField (Vector a) where
+    fromField f v = V.fromList . fromPGArray <$> fromField f v
 
-mkCompats :: [BuiltinType] -> Compat
-mkCompats = foldl' f (Compat 0) . map mkCompat
-  where f (Compat a) (Compat b) = Compat (a .|. b)
+instance (FromField a, Typeable a) => FromField (IOVector a) where
+    fromField f v = liftConversion . V.unsafeThaw =<< fromField f v
 
-mkCompat :: BuiltinType -> Compat
-mkCompat = Compat . shiftL 1 . fromEnum
+-- | uuid
+instance FromField UUID where
+    fromField f mbs =
+      if typeOid f /= $(inlineTypoid TI.uuid)
+      then returnError Incompatible f ""
+      else case mbs of
+             Nothing -> returnError UnexpectedNull f ""
+             Just bs ->
+                 case UUID.fromASCIIBytes bs of
+                   Nothing -> returnError ConversionFailed f "Invalid UUID"
+                   Just uuid -> pure uuid
 
-compat :: Compat -> Compat -> Bool
-compat (Compat a) (Compat b) = a .&. b /= 0
+-- | json
+instance FromField JSON.Value where
+    fromField f mbs =
+      if typeOid f /= $(inlineTypoid TI.json)
+      then returnError Incompatible f ""
+      else case mbs of
+             Nothing -> returnError UnexpectedNull f ""
+             Just bs ->
+#if MIN_VERSION_aeson(0,6,3)
+                 case JSON.eitherDecodeStrict' bs of
+#elif MIN_VERSION_bytestring(0,10,0)
+                 case JSON.eitherDecode' $ LB.fromStrict bs of
+#else
+                 case JSON.eitherDecode' $ LB.fromChunks [bs] of
+#endif
+                   Left  err -> returnError ConversionFailed f err
+                   Right val -> pure val
+
+-- | Parse a field to a JSON 'JSON.Value' and convert that into a
+-- Haskell value using 'JSON.fromJSON'.
+--
+-- This can be used as the default implementation for the 'fromField'
+-- method for Haskell types that have a JSON representation in
+-- PostgreSQL.
+--
+-- The 'Typeable' constraint is required to show more informative
+-- error messages when parsing fails.
+fromJSONField :: (JSON.FromJSON a, Typeable a) => FieldParser a
+fromJSONField f mbBs = do
+    value <- fromField f mbBs
+    case JSON.fromJSON value of
+        JSON.Error err -> returnError ConversionFailed f $
+                            "JSON decoding error: " ++ err
+        JSON.Success x -> pure x
+
+
+-- | Compatible with the same set of types as @a@.  Note that
+--   modifying the 'IORef' does not have any effects outside
+--   the local process on the local machine.
+instance FromField a => FromField (IORef a) where
+    fromField f v = liftConversion . newIORef =<< fromField f v
+
+-- | Compatible with the same set of types as @a@.  Note that
+--   modifying the 'MVar' does not have any effects outside
+--   the local process on the local machine.
+instance FromField a => FromField (MVar a) where
+    fromField f v = liftConversion . newMVar =<< fromField f v
+
+type Compat = PQ.Oid -> Bool
 
 okText, okText', okBinary, ok16, ok32, ok64, okInt :: Compat
-okText   = mkCompats [Name,Text,Char,BpChar,VarChar]
-okText'  = mkCompats [Name,Text,Char,BpChar,VarChar,Unknown]
-okBinary = mkCompats [ByteA]
-ok16 = mkCompats [Int2]
-ok32 = mkCompats [Int2,Int4]
-ok64 = mkCompats [Int2,Int4,Int8]
+okText   = $( mkCompats [ TI.name, TI.text, TI.char,
+                          TI.bpchar, TI.varchar ] )
+okText'  = $( mkCompats [ TI.name, TI.text, TI.char,
+                          TI.bpchar, TI.varchar, TI.unknown ] )
+okBinary = (== $( inlineTypoid TI.bytea ))
+ok16 = (== $( inlineTypoid TI.int2 ))
+ok32 = $( mkCompats [TI.int2,TI.int4] )
+ok64 = $( mkCompats [TI.int2,TI.int4,TI.int8] )
 #if WORD_SIZE_IN_BITS < 64
 okInt = ok32
 #else
@@ -462,9 +534,8 @@ okInt = ok64
 doFromField :: forall a . (Typeable a)
           => Field -> Compat -> (ByteString -> Conversion a)
           -> Maybe ByteString -> Conversion a
-doFromField f types cvt (Just bs)
-    | Just typ <- oid2builtin (typeOid f)
-    , mkCompat typ `compat` types = cvt bs
+doFromField f isCompat cvt (Just bs)
+    | isCompat (typeOid f) = cvt bs
     | otherwise = returnError Incompatible f "types incompatible"
 doFromField f _ _ _ = returnError UnexpectedNull f ""
 
