@@ -1,4 +1,4 @@
-{-# LANGUAGE RecordWildCards #-}
+{-# LANGUAGE RecordWildCards, FlexibleInstances #-}
 
 ------------------------------------------------------------------------------
 -- |
@@ -12,7 +12,10 @@
 -- returned by a SQL query into a more useful Haskell representation.
 --
 -- Predefined instances are provided for tuples containing up to ten
--- elements.
+-- elements.  The instances for 'Maybe' types return 'Nothing' if all
+-- the columns that would have been otherwise consumed are null,  otherwise
+-- it attempts a regular conversion.
+--
 ------------------------------------------------------------------------------
 
 module Database.PostgreSQL.Simple.FromRow
@@ -23,23 +26,25 @@ module Database.PostgreSQL.Simple.FromRow
      , numFieldsRemaining
      ) where
 
-import Control.Applicative (Applicative(..), (<$>))
-import Control.Monad (replicateM)
-import Data.ByteString (ByteString)
+import           Prelude hiding (null)
+import           Control.Applicative (Applicative(..), (<$>), (<|>), (*>))
+import           Control.Monad (replicateM, replicateM_)
+import           Control.Monad.Trans.State.Strict
+import           Control.Monad.Trans.Reader
+import           Control.Monad.Trans.Class
+import           Data.ByteString (ByteString)
 import qualified Data.ByteString.Char8 as B
-import Database.PostgreSQL.Simple.Types (Only(..))
+import           Data.Vector (Vector)
+import qualified Data.Vector as V
+import           Database.PostgreSQL.Simple.Types (Only(..))
 import qualified Database.PostgreSQL.LibPQ as PQ
 import           Database.PostgreSQL.Simple.Internal
 import           Database.PostgreSQL.Simple.Compat
 import           Database.PostgreSQL.Simple.FromField
 import           Database.PostgreSQL.Simple.Ok
-import           Database.PostgreSQL.Simple.Types ((:.)(..))
+import           Database.PostgreSQL.Simple.Types ((:.)(..), Null)
 import           Database.PostgreSQL.Simple.TypeInfo
 
-import Control.Monad.Trans.State.Strict
-import Control.Monad.Trans.Reader
-import Control.Monad.Trans.Class
-
 -- | A collection type that can be converted from a sequence of fields.
 -- Instances are provided for tuples up to 10 elements and lists of any length.
 --
@@ -57,14 +62,15 @@ import Control.Monad.Trans.Class
 -- exception will be thrown.
 --
 -- Note that 'field' evaluates it's result to WHNF, so the caveats listed in
--- previous versions of postgresql-simple no longer apply.  Instead, look
--- at the caveats associated with user-defined implementations of 'fromRow'.
+-- mysql-simple and very early versions of postgresql-simple no longer apply.
+-- Instead, look at the caveats associated with user-defined implementations
+-- of 'fromField'.
 
 class FromRow a where
     fromRow :: RowParser a
 
 getvalue :: PQ.Result -> PQ.Row -> PQ.Column -> Maybe ByteString
-getvalue result row col = unsafeDupablePerformIO (PQ.getvalue result row col)
+getvalue result row col = unsafeDupablePerformIO (PQ.getvalue' result row col)
 
 nfields :: PQ.Result -> PQ.Column
 nfields result = unsafeDupablePerformIO (PQ.nfields result)
@@ -117,57 +123,132 @@ numFieldsRemaining = RP $ do
     column <- lift get
     return $! (\(PQ.Col x) -> fromIntegral x) (nfields rowresult - column)
 
+null :: RowParser Null
+null =  field
+
 instance (FromField a) => FromRow (Only a) where
     fromRow = Only <$> field
 
+instance (FromField a) => FromRow (Maybe (Only a)) where
+    fromRow =  (null *> pure Nothing)
+           <|> (Just <$> fromRow)
+
 instance (FromField a, FromField b) => FromRow (a,b) where
     fromRow = (,) <$> field <*> field
 
+instance (FromField a, FromField b) => FromRow (Maybe (a,b)) where
+    fromRow =  (null *> null *> pure Nothing)
+           <|> (Just <$> fromRow)
+
 instance (FromField a, FromField b, FromField c) => FromRow (a,b,c) where
     fromRow = (,,) <$> field <*> field <*> field
 
+instance (FromField a, FromField b, FromField c) => FromRow (Maybe (a,b,c)) where
+    fromRow =  (null *> null *> null *> pure Nothing)
+           <|> (Just <$> fromRow)
+
 instance (FromField a, FromField b, FromField c, FromField d) =>
     FromRow (a,b,c,d) where
     fromRow = (,,,) <$> field <*> field <*> field <*> field
 
+instance (FromField a, FromField b, FromField c, FromField d) =>
+    FromRow (Maybe (a,b,c,d)) where
+    fromRow =  (null *> null *> null *> null *> pure Nothing)
+           <|> (Just <$> fromRow)
+
 instance (FromField a, FromField b, FromField c, FromField d, FromField e) =>
     FromRow (a,b,c,d,e) where
     fromRow = (,,,,) <$> field <*> field <*> field <*> field <*> field
 
+instance (FromField a, FromField b, FromField c, FromField d, FromField e) =>
+    FromRow (Maybe (a,b,c,d,e)) where
+    fromRow =  (null *> null *> null *> null *> null *> pure Nothing)
+           <|> (Just <$> fromRow)
+
 instance (FromField a, FromField b, FromField c, FromField d, FromField e,
           FromField f) =>
     FromRow (a,b,c,d,e,f) where
     fromRow = (,,,,,) <$> field <*> field <*> field <*> field <*> field
                       <*> field
 
+instance (FromField a, FromField b, FromField c, FromField d, FromField e,
+          FromField f) =>
+    FromRow (Maybe (a,b,c,d,e,f)) where
+    fromRow =  (null *> null *> null *> null *> null *>
+                null *> pure Nothing)
+           <|> (Just <$> fromRow)
+
 instance (FromField a, FromField b, FromField c, FromField d, FromField e,
           FromField f, FromField g) =>
     FromRow (a,b,c,d,e,f,g) where
     fromRow = (,,,,,,) <$> field <*> field <*> field <*> field <*> field
                        <*> field <*> field
 
+instance (FromField a, FromField b, FromField c, FromField d, FromField e,
+          FromField f, FromField g) =>
+    FromRow (Maybe (a,b,c,d,e,f,g)) where
+    fromRow =  (null *> null *> null *> null *> null *>
+                null *> null *> pure Nothing)
+           <|> (Just <$> fromRow)
+
 instance (FromField a, FromField b, FromField c, FromField d, FromField e,
           FromField f, FromField g, FromField h) =>
     FromRow (a,b,c,d,e,f,g,h) where
     fromRow = (,,,,,,,) <$> field <*> field <*> field <*> field <*> field
                         <*> field <*> field <*> field
 
+instance (FromField a, FromField b, FromField c, FromField d, FromField e,
+          FromField f, FromField g, FromField h) =>
+    FromRow (Maybe (a,b,c,d,e,f,g,h)) where
+    fromRow =  (null *> null *> null *> null *> null *>
+                null *> null *> null *> pure Nothing)
+           <|> (Just <$> fromRow)
+
 instance (FromField a, FromField b, FromField c, FromField d, FromField e,
           FromField f, FromField g, FromField h, FromField i) =>
     FromRow (a,b,c,d,e,f,g,h,i) where
     fromRow = (,,,,,,,,) <$> field <*> field <*> field <*> field <*> field
                          <*> field <*> field <*> field <*> field
 
+instance (FromField a, FromField b, FromField c, FromField d, FromField e,
+          FromField f, FromField g, FromField h, FromField i) =>
+    FromRow (Maybe (a,b,c,d,e,f,g,h,i)) where
+    fromRow =  (null *> null *> null *> null *> null *>
+                null *> null *> null *> null *> pure Nothing)
+           <|> (Just <$> fromRow)
+
 instance (FromField a, FromField b, FromField c, FromField d, FromField e,
           FromField f, FromField g, FromField h, FromField i, FromField j) =>
     FromRow (a,b,c,d,e,f,g,h,i,j) where
     fromRow = (,,,,,,,,,) <$> field <*> field <*> field <*> field <*> field
                           <*> field <*> field <*> field <*> field <*> field
 
+instance (FromField a, FromField b, FromField c, FromField d, FromField e,
+          FromField f, FromField g, FromField h, FromField i, FromField j) =>
+    FromRow (Maybe (a,b,c,d,e,f,g,h,i,j)) where
+    fromRow =  (null *> null *> null *> null *> null *>
+                null *> null *> null *> null *> null *> pure Nothing)
+           <|> (Just <$> fromRow)
+
 instance FromField a => FromRow [a] where
     fromRow = do
       n <- numFieldsRemaining
       replicateM n field
 
+instance FromField a => FromRow (Maybe [a]) where
+    fromRow = do
+      n <- numFieldsRemaining
+      (replicateM_ n null *> pure Nothing) <|> (Just <$> replicateM n field)
+
+instance FromField a => FromRow (Vector a) where
+    fromRow = do
+      n <- numFieldsRemaining
+      V.replicateM n field
+
+instance FromField a => FromRow (Maybe (Vector a)) where
+    fromRow = do
+      n <- numFieldsRemaining
+      (replicateM_ n null *> pure Nothing) <|> (Just <$> V.replicateM n field)
+
 instance (FromRow a, FromRow b) => FromRow (a :. b) where
     fromRow = (:.) <$> fromRow <*> fromRow
