@@ -44,6 +44,7 @@ import           Database.PostgreSQL.Simple.Types (Query(..))
 import           Database.PostgreSQL.Simple.TypeInfo.Types(TypeInfo)
 import           Control.Monad.Trans.State.Strict
 import           Control.Monad.Trans.Reader
+import           Control.Monad.Trans.Class
 import           GHC.IO.Exception
 
 -- | A Field represents metadata about a particular field
@@ -296,8 +297,14 @@ data Row = Row {
 newtype RowParser a = RP { unRP :: ReaderT Row (StateT PQ.Column Conversion) a }
    deriving ( Functor, Applicative, Alternative, Monad )
 
+liftRowParser :: IO a -> RowParser a
+liftRowParser = RP . lift . lift . liftConversion
+
 newtype Conversion a = Conversion { runConversion :: Connection -> IO (Ok a) }
 
+liftConversion :: IO a -> Conversion a
+liftConversion m = Conversion (\_ -> Ok <$> m)
+
 instance Functor Conversion where
    fmap f m = Conversion $ \conn -> (fmap . fmap) f (runConversion m conn)
 
