@@ -18,12 +18,14 @@ module Database.PostgreSQL.Simple.ToField
     (
       Action(..)
     , ToField(..)
+    , toJSONField
     , inQuotes
     ) where
 
 import Blaze.ByteString.Builder (Builder, fromByteString, toByteString)
 import Blaze.ByteString.Builder.Char8 (fromChar)
 import Blaze.Text (integral, double, float)
+import qualified Data.Aeson as JSON
 import Data.ByteString (ByteString)
 import Data.Int (Int8, Int16, Int32, Int64)
 import Data.List (intersperse)
@@ -31,13 +33,15 @@ import Data.Monoid (mappend)
 import Data.Time (Day, TimeOfDay, LocalTime, UTCTime, ZonedTime)
 import Data.Typeable (Typeable)
 import Data.Word (Word, Word8, Word16, Word32, Word64)
-import Database.PostgreSQL.Simple.Types (Binary(..), In(..), Null)
+import Database.PostgreSQL.Simple.Types
 import qualified Blaze.ByteString.Builder.Char.Utf8 as Utf8
 import qualified Data.ByteString as SB
 import qualified Data.ByteString.Lazy as LB
 import qualified Data.Text as ST
 import qualified Data.Text.Encoding as ST
 import qualified Data.Text.Lazy as LT
+import           Data.UUID   (UUID)
+import qualified Data.UUID as UUID
 import           Data.Vector (Vector)
 import qualified Data.Vector as V
 import qualified Database.PostgreSQL.LibPQ as PQ
@@ -95,6 +99,10 @@ instance ToField Null where
     toField _ = renderNull
     {-# INLINE toField #-}
 
+instance ToField Default where
+    toField _ = Plain (fromByteString "default")
+    {-# INLINE toField #-}
+
 instance ToField Bool where
     toField True  = Plain (fromByteString "true")
     toField False = Plain (fromByteString "false")
@@ -222,14 +230,32 @@ instance ToField Date where
     toField = Plain . inQuotes . dateToBuilder
     {-# INLINE toField #-}
 
-instance (ToField a) => ToField (Vector a) where
+instance (ToField a) => ToField (PGArray a) where
     toField xs = Many $
         Plain (fromByteString "ARRAY[") :
-        (intersperse (Plain (fromChar ',')) . map toField $ V.toList xs) ++
+        (intersperse (Plain (fromChar ',')) . map toField $ fromPGArray xs) ++
         [Plain (fromChar ']')]
         -- Because the ARRAY[...] input syntax is being used, it is possible
         -- that the use of type-specific separator characters is unnecessary.
 
+instance (ToField a) => ToField (Vector a) where
+    toField = toField . PGArray . V.toList
+
+instance ToField UUID where
+    toField = Plain . inQuotes . fromByteString . UUID.toASCIIBytes
+
+instance ToField JSON.Value where
+    toField = toField . JSON.encode
+
+-- | Convert a Haskell value to a JSON 'JSON.Value' using
+-- 'JSON.toJSON' and convert that to a field using 'toField'.
+--
+-- This can be used as the default implementation for the 'toField'
+-- method for Haskell types that have a JSON representation in
+-- PostgreSQL.
+toJSONField :: JSON.ToJSON a => a -> Action
+toJSONField = toField . JSON.toJSON
+
 -- | Surround a string with single-quote characters: \"@'@\"
 --
 -- This function /does not/ perform any other escaping.
