@@ -16,6 +16,7 @@
 module Database.PostgreSQL.Simple.Types
     (
       Null(..)
+    , Default(..)
     , Only(..)
     , In(..)
     , Binary(..)
@@ -23,6 +24,7 @@ module Database.PostgreSQL.Simple.Types
     , Oid(..)
     , (:.)(..)
     , Savepoint(..)
+    , PGArray(..)
     ) where
 
 import Blaze.ByteString.Builder (toByteString)
@@ -43,6 +45,10 @@ instance Eq Null where
     _ == _ = False
     _ /= _ = False
 
+-- | A placeholder for the PostgreSQL @DEFAULT@ value.
+data Default = Default
+           deriving (Read, Show, Typeable)
+
 -- | A query string. This type is intended to make it difficult to
 -- construct a SQL query by concatenating string fragments, as that is
 -- an extremely common way to accidentally introduce SQL injection
@@ -109,7 +115,12 @@ newtype In a = In a
     deriving (Eq, Ord, Read, Show, Typeable, Functor)
 
 -- | Wrap binary data for use as a @bytea@ value.
-newtype Binary a = Binary a
+newtype Binary a = Binary {fromBinary :: a}
+    deriving (Eq, Ord, Read, Show, Typeable, Functor)
+
+
+-- | Wrap a list for use as a PostgreSQL array.
+newtype PGArray a = PGArray {fromPGArray :: [a]}
     deriving (Eq, Ord, Read, Show, Typeable, Functor)
 
 -- | A composite type to parse your custom data structures without
